import json

from portal.database.models import Course, Project
from portal.service import courses, general
from . import utils


def test_list(client):
    response = utils.make_request(client, '/courses', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    courses = utils.extract_data(response)
    assert len(courses) == 2
    db_courses = Course.query.all()
    utils.assert_course_in(db_courses, courses[0])


def test_create(client):
    request_dict = dict(
        name="java",
        codename="PA165",
        )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/courses', data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_course = utils.extract_data(response)
    db_courses = Course.query.all()
    assert len(db_courses) == db_courses_number + 1
    utils.assert_course_in(db_courses, resp_new_course)


def test_create_extra_key(client):
    request_dict = dict(
        name="java",
        codename="PA165",
        extra="should not be here"
        )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/courses', data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    resp_new_course = utils.extract_data(response)
    db_courses = Course.query.all()
    assert len(db_courses) == db_courses_number + 1
    utils.assert_course_in(db_courses, resp_new_course)


def test_create_missing_key(client):
    request_dict = dict(
        codename="PA165",
        )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/courses', data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'
    assert len(Course.query.all()) == db_courses_number


def test_create_invalid_value(client):
    request_dict = dict(
        name=456,
        codename="PA165",
        )
    db_courses_number = len(Course.query.all())

    request_json = json.dumps(request_dict)
    response = utils.make_request(client, '/courses', data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='post')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'
    assert len(Course.query.all()) == db_courses_number


def test_read(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = utils.make_request(client, f"/courses/{c.id}", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    course = utils.extract_data(response)

    utils.assert_course(c, course)


def test_update(client):
    c = Course.query.filter_by(codename="PB161").first()
    request_dict = dict(
        name="new_cpp",
        codename="PB161_new",
        )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f'/courses/{c.id}', data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    assert not Course.query.filter_by(codename="PB161").first()
    course_from_db = Course.query.filter_by(codename="PB161_new").first()
    assert course_from_db.name == "new_cpp"


def test_delete(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = utils.make_request(client, f'/courses/{c.id}',
                                  headers={"content-type": "application/json"},
                                  method='delete')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    assert not Course.query.filter_by(codename="PB161").first()


def test_notes_token_read(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = utils.make_request(client, f"/courses/{c.id}/notes_access_token", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = utils.extract_data(response)
    assert data == "PB161_token"


def test_notes_token_create(client):
    c = Course.query.filter_by(codename="PB071").first()
    response = utils.make_request(client, f"/courses/{c.id}/notes_access_token",
                                  data=json.dumps("new_token"),
                                  headers={"content-type": "application/json"}, method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    updated = Course.query.filter_by(codename="PB071").first()
    assert updated.notes_access_token == "new_token"


def test_notes_token_update(client):
    c = Course.query.filter_by(codename="PB161").first()
    response = utils.make_request(client, f"/courses/{c.id}/notes_access_token",
                                  data=json.dumps("new_token"),
                                  headers={"content-type": "application/json"}, method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    updated = Course.query.filter_by(codename="PB161").first()
    assert updated.notes_access_token == "new_token"


def test_copy_from_course(client):
    source: Course = general.find_course('PB161')
    target = courses.create_course(name='test_course', codename='test')
    request_dict = {
        "source_course": "PB161",
        "config": {
            "roles": "with_users",
            "groups": "with_users",
            "projects": "bare",
            }
        }
    request_json = json.dumps(request_dict)

    response = utils.make_request(client, f'/courses/{target.codename}/import',
                                  data=request_json,
                                  headers={"content-type": "application/json"},
                                  method='put')
    assert response.status_code == 204
    assert response.mimetype == "application/json"

    updated_course: Course = general.find_course(target.codename)

    print(f"CPP projects:{source.projects}\n Target projects:{updated_course.projects}\n\n")
    print(f"CPP roles:{source.roles}\n Target roles:{updated_course.roles}\n\n")
    print(f"CPP groups:{source.groups}\n Target groups:{updated_course.groups}\n\n")

    assert_collections(source, updated_course, 'projects')
    assert_collections(source, updated_course, 'roles')
    assert_collections(source, updated_course, 'groups')


def assert_collections(expected: Course, actual: Course, collection: str):
    def extract_names(obj, coll_name: str):
        coll = getattr(obj, coll_name)
        return [elem.name for elem in coll]

    expected_names = extract_names(expected, collection)
    actual_names = extract_names(actual, collection)
    for elem in actual_names:
        assert elem in expected_names


'''
* [POST] /courses/<cid>/archive : archives a course, all associated groups, projects, roles
	- requires a new codename
	- request:
	```json
		{
			"archive_as":"PB161_2017"
		}
    ```

'''
